//	MsgThr.hpp

#ifndef _MSGTHR_HPP_
#define _MSGTHR_HPP_

#include "wpch.hpp"			// for Power++ thread library
#include "BCirQpov.hpp"		// for queue stuff

//////////////////////////////////////////////////////////////////////////////
//
//	class	Processor
//
//	Abstract base class. Provides the interface reqd by MsgThreadW objects.
//
//////////////////////////////////////////////////////////////////////////////
//
template<class T>
class	Processor
{
public:
	virtual void	Process( T* _msg ) = 0;
};


//////////////////////////////////////////////////////////////////////////////
//
//	class	MsgThreadW
//
//	This class requires the Power++ thread library. As such it is only
//	useable with Power++ to create Win32 apps.
//
//	Implements a worker thread with a message queue.
//	Queue elements must be ptrs to dynamically created objects.
//
//	Instantiantiating a MsgThreadW object will create a new thread which
// 	is devoted to processing messages sent to it.
// 	Messages are automatically removed from the queue as fast as possible & 
//	sent to the processor object, by calling it's Process() method. 
//  The process thread will sleep until entries are added to the message queue,
//	ie by calling the Add() method. This ensures that the thread will not waste
// 	CPU time if there are no messages in the queue.
//
//	The processor object must be derived from class Processor & must override the
//	Process() pure virtual method. 	
//
//	The thread priority can be set by passing the required setting to the constructor.
//	The default setting is WThreadPriorityNormal.
//	Allowable settings are : 
//		WThreadPriorityLowest, 	WThreadPriorityIdle, 	WThreadPriorityBelowNormal
//		WThreadPriorityNormal, WThreadPriorityAboveNormal, WThreadPriorityHighest,
//		WThreadPriorityTimeCritical
//
//	The size of the message queue can also be set in the constructor.
//	If no size is specified the queue will hold 50 messages.
//
//	void	Add( Message* _msg )
//		Adds a message to the message queue.
//
//	bool	Running()
//		Returns true if the process thread is running, else returns false
//
//////////////////////////////////////////////////////////////////////////////
//
template<class T>
class	MsgThreadW : public WObject
{
public:
	MsgThreadW( Processor<T>* _prcsr, WThreadPriority _priority=WThreadPriorityNormal, uint _queSize=50 )
		: msgQue(_queSize), processor(_prcsr), running(true) 
	{
        grabQue.Create( "grabQue" );		// create the mutex semaphore
		datainQueue.Create( "msgQue" );		// create the event semaphore
		datainQueue.Reset();				// make the process thread sleep
		if ( 0 == processor )
			running = false;	// don't have a valid procesor, so shutdown
		else
		{
			// have a valid form so do all the set up stuff
			processThread.Create( this, (WThreadRoutine)Process, 0 );
			processThread.SetPriority( _priority );
		}
	}
	
	~MsgThreadW()
	{
		running = false;		// tell thread to shutdown
	    datainQueue.Set();      // stop thread waiting on semaphore
		processThread.Wait();	// wait for thread to shutdown
	}
	
	void	Add( Message* _msg )
	{ 
		if ( 0 !=_msg )
		{
	        grabQue.Acquire();		// lock the queue for our use
			msgQue.Add( _msg ); 	// add message to the queue
			datainQueue.Set(); 		// Wake up the Process thread
	        grabQue.Release();		// free the queue for others to use
		}
	}
	
	bool	Running() const	{ return running; }
	
private:
	WDWord	Process( void* )
	{
		while ( running )
		{
			datainQueue.Wait();			// sleep until have messages to process
	        if ( msgQue.Count() == 0 )
	            datainQueue.Reset();	// Queue is empty, so put thread to sleep
	        else
	        	// have messages, so get them processed.
			    processor->Process( msgQue.Remove() );
		}
		return 0;
	}
	
	BQueuePtrOvr<Message>	msgQue;			// the message queue container
	Processor<T>*			processor;		// the processor object
	WThread					processThread;	// the thread used to do the processing
	bool					running;		// true if process thread is running
	WEventSemaphore 		datainQueue;	// set if messages in queue, else reset
    WMutexSemaphore         grabQue;		// coordinates multi thread access to Add()
};

#endif	// _MSGTHR_HPP_
