//	BCircQ_po.hpp
//
//  Declaration for BCircQ_po class

#ifndef __BCIRQ_PO__HPP__
#define __BCIRQ_PO__HPP__

#include "BQueueP.hpp"
#include "Bcirq_o.hpp"

#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class BCircQ_po
//
//  Implements a simple bounded circular queue for pointers, allowing overwrites
//	Each element added to the queue is a ptr to a block of dynamically alloc memory.
//
//	The queue store size is fixed (bounded) when it is created.
//	Attempting to over fill the queue will overwrite existing data
//	Attempting to remove data from an empty queue will throw a QueueException
//
//  Public Methods:
//	---------------
//  constructor     
//		Accepts 1 arg which specifies the queue size.
//      Dynamically allocates memory for the queue buffer.

//  destructor      
//		Deletes memory pointed at by elements in the queue
//		Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added.
//		If queue has no free space, 1st element in queue will be overwritten

//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		If _qty is > free space in queue, data will be overwritten.
//		EG 1 free space = 10, _qty = 15, existing queue elmnts will be overwritten
//		EG 2 capacity = 10, _qty = 15, only last 10 elements in _src will be stored
//		Returns no. of elements added, does NOT throw QueueException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a QueueException
//
// T   		Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// T  		ReadX( const uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//		Wrapper for Read(). Can be called with constant for _pos
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//		Wrapper for Read(). Can be called with constant for _pos
//
// void	Empty()			//		Empties the queue.
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
template<class T>
class   BCircQ_po : public BCircQ_o<T const *>
{
public:
    BCircQ_po( uint _size ) : BCircQ_o(_size) {}
    
    virtual ~BCircQ_po()  { Empty(); }
    
    T const *    	Add( T const * _item ) 
    { 
    	if ( Count() == Capacity() )
    	{
    		// are about to overwrite 1st element, so make sure release
    		// memory it points to before destroying the element.
    		delete (T*) BCircQ_o::Remove();
    	}
    	return BCircQ_o::Add( _item ); 
    }
    
    uint	Add( T const *  const * _src, uint _qty ) 
    { 
    	if ( FreeSpace() < _qty && Count() > 0 )
    	{
    		// are about to overwrite existing elements
    		// so free the memory they point to before overwiting them.
    		
    		// remQty is set to the no. of elements that will be overwritten
    		// Note: remQty must be <= Count()
    		uint	remQty = _qty - FreeSpace();
    		remQty = remQty > Count() ? Count() : remQty;
    		for ( uint i=0; remQty > i; i++ )
    			delete (T*) BCircQ_o::Remove();
    	}
    	return BCircQ_o::Add( _src, _qty ); 
    }

	void		Empty() 
    {
		try
		{
			while ( true )
			{
				T const * 	el = Remove();
			 	delete (T*) el;
			}
		}
		catch ( QueueException _ex )
		{
		}
    }
    
};


#ifdef USING
}		// namespace ktUtil
#endif

#endif  // __BCIRQ_PO__HPP__
