//	BCirQpov.hpp
//
//  Declaration for BQueuePtrOvr class

#ifndef __BCIRQPOV__HPP__
#define __BCIRQPOV__HPP__

#include "BQueueP.hpp"
#include "Bcirq_po.hpp"

#ifdef USING
namespace ktUtil
{
#endif

//===========================================================================//
//  class BQueuePtrOvr
//  Implements a simple circular queue, for pointers, allowing overwrites
//
//	Attempting to over fill the queue will overwrite existing data
//	Attempting to remove data from an empty queue will throw a QueueException
//
//  Public Methods:
//	---------------
//  constructor     Accepts 1 arg which specifies the queue size.
//                  Dynamically allocates memory for the queue buffer.
//  destructor      Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added.
//		If queue has no free space, 1st element in queue will be overwritten
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		If _qty is > free space in queue, data will be overwritten.
//		EG 1 free space = 10, _qty = 15, existing queue elmnts will be overwritten
//		EG 2 capacity = 10, _qty = 15, only last 10 elements in _src will be stored
//		Returns no. of elements added, does NOT throw QueueException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a QueueException
//
// T   		Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// T  		ReadX( const uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//		Wrapper for Read(). Can be called with constant for _pos
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//		Wrapper for Read(). Can be called with constant for _pos
//
// void		Empty()		//	Empties the queue.
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
template<class T>
class   BQueuePtrOvr : public BQueuePtr<T>, private BCircQ_po<T>
{
public:
    BQueuePtrOvr( uint _size ) : BCircQ_po(_size) {}
    virtual ~BQueuePtrOvr() {}
    
    virtual T const *	Add( T const * _item ) { return BCircQ_po::Add( _item ); }
    virtual uint		Add( T const *  const * _src, uint _qty ) { return BCircQ_po::Add( _src, _qty ); }

    virtual T const *	Remove() { return BCircQ_po::Remove(); }
    virtual uint   		Remove( T const * * _dest, uint _qty ) { return BCircQ_po::Remove( _dest, _qty ); }

    virtual T const *    	Read( uint& _pos ) { return BCircQ_po::Read( _pos ); }
    virtual T const *	    ReadX( const uint& _pos ) { return BCircQ_po::ReadX( _pos ); }
    virtual uint   	Read( T const ** _dest, uint _qty, uint& _pos ) { return BCircQ_po::Read( _dest, _qty, _pos ); }
    virtual uint   	ReadX( T const ** _dest, uint _qty, const uint& _pos ) { return BCircQ_po::ReadX( _dest, _qty, _pos ); }
	virtual void	Empty() { BCircQ_po::Empty(); }

	virtual uint	Count( uint _pos ) const { return BCircQ_po::Count( _pos ); }
	virtual uint	Count()	const { return BCircQ_po::Count(); }
	virtual uint	Capacity()	const { return BCircQ_po::Capacity(); }
	virtual uint	FreeSpace() const { return BCircQ_po::FreeSpace(); }
	virtual uint	End() const { return BCircQ_po::End(); }
	virtual uint	Begin() const { return BCircQ_po::Start(); }
};


#ifdef USING
}		// namespace ktUtil
#endif

#endif  // __BCIRQPOV__HPP__
