//  Bcirq_o.hpp
//
//  Declaration for BCircQ_o class

#ifndef __BCIRCQ_O__HPP__
#define __BCIRCQ_O__HPP__

#include "CircCntr.hpp"
#include "QueEx.hpp"

#ifndef  uint
typedef unsigned int  uint;
#endif

#ifdef USING
namespace ktUtil
{
#endif


//class	CircCntr;			// forward declaration
//class	QueueException;		// forward declaration

//===========================================================================//
//  class BCircQ_o
//
//  Implements a simple bounded circular queue, allowing overwrites
//	The queue store size is fixed (bounded) when it is created.
//	Attempting to over fill the queue will overwrite existing data
//	Attempting to remove data from an empty queue will throw a QueueException
//
//  Public Methods:
//	---------------
//  constructor     
//		Accepts 1 arg which specifies the queue size.
//      Dynamically allocates memory for the queue buffer.
// 		If unable to allocate memory for queue store throws a QueueException
//  destructor      
//		Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added.
//		If queue is full, 1st element in queue will be overwritten
//
// void	 	Add( T const * _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		If _qty is > free space in queue, data will be overwritten.
//		EG 1 free space = 10, _qty = 15, existing queue elmnts will be overwritten
//		EG 2 capacity = 10, _qty = 15, only last 10 elements in _src will be stored
//		Returns no. of elements added, does NOT throw QueueException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a QueueException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a QueueException
//
// const T&	Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// const T&	ReadX( const uint& _pos )
//		Wrapper for Read(). Can be called with constant for _pos
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a QueueException
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
//		Wrapper for Read(). Can be called with constant for _pos
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//
// void	Empty()			//		Empties the queue.
//
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
template<class T>
class   BCircQ_o
{
public:
    BCircQ_o( uint _size )
		: count(0), queSize(0), start(_size), end(_size)
	{
		#ifdef newalloc
			try
			{
				queue = new T[_size];
			}
			catch ( bad_alloc& _ex )
			{
				throw QueueException(QueueException::NoQueueMem);
			}
			queSize = _size;
		#else	
			queue = new T[_size];
			if ( queue )
				queSize = _size;
			else
				// unable to allocate memory for the queue
				throw QueueException(QueueException::NoQueueMem);	
		#endif	
	}

    virtual ~BCircQ_o()
    {
		delete [] queue;
	}

    
    const T&    Add( T const & _item )
    //
	// Adds 1 element to the queue
	// This may overwrite existing queue data
	//
	{
		queue[end++] = _item;
		++count;	
		if ( count > queSize )
		{
			count = queSize;
			start = end;
		}
		return _item;
	}

    uint	    Add( T const * _src, uint _qty )
    //
	// Adds _qty elements from _src into the queue
	// This may overwrite existing queue data
	// Returns actual no. of elements added, which is always no. specified.
	//
	{
		if ( _qty > queSize )
		{
			// trying to add more than queue can hold.
			// So just add enough to fill the queue, rest are 'overwritten'
			start = end = 0;
			count = queSize;
			::memmove( queue, _src + _qty - queSize, queSize * sizeof(T) );
			return _qty;
		}
		// if get here queue can fit _qty elements
		uint	tmp = queSize - end;
		if ( tmp >= _qty )
		{
		 	// new elements fit between end ptr & actual end of queue array
		 	::memmove( queue+end, _src, _qty * sizeof(T) );
		}
		else
		{
			// fill tmp elements to physical end of array
		 	::memmove( queue+end, _src, tmp * sizeof(T) );
		 	// then fill rest from physicalstart of array
		 	::memmove( queue, _src+tmp, (_qty-tmp) * sizeof(T) );
		}
		count += _qty;	// increment count to reflect added elements
		end += _qty;	// move end ptr to next 'free' space
		// now make sure that attribs are valid
		if ( count > queSize )
		{
			count = queSize;
			start = end;
		}
		return _qty;	
	}

    const T&   	Remove()
    //
	// remove 1 element from the queue
	// If queue is empty throws a QueueException
	//
	{
		if ( 0 >= count )
			throw QueueException(QueueException::EmptyQ);		// empty queue
		--count;
		return queue[start++];
	}

    uint   		Remove( T* _dest, uint _qty )
    //
	// Removes up to _qty elements from the queue into _dest
	// Returns actual no. of elements removed.
	//
	{
		_qty =	ReadX( _dest, _qty, 0 );	// copy elmnts from start of queue
		
		// now adjust queue start & count to 'remove' elmnts just copied
		start += _qty;	// increment start ptr to new start of queue data
		count -= _qty;	// decrement the count of elements in the queue	
	
		return _qty;	// return no. of elmnts removed
	}

    const T&    Read( uint& _pos )
    //
	// Returns copy of elmnt at posn _pos in queue
	// Updates _pos to indicate next element (if any) in queue.
	// Does Not wrap around past end of queue.
	// If _posn indicates element outside queue, throws a QueueException
	//
	{
		if ( _pos >= count /*|| _pos == end*/ )
			throw QueueException(QueueException::OutsideQ);		// invalid request
		
		CircCntr	posn( start, _pos++ );
	
		return queue[posn];
	}
	
    const T&    ReadX( const uint& _pos )
    //
	// Returns copy of elmnt at posn _pos in queue
	// Does Not wrap around past end of queue.
	// If _posn indicates element outside queue, throws a QueueException
	//
	{
		uint	pos = _pos;
		return Read( pos );
	}

    uint   		Read( T* _dest, uint _qty, uint& _pos )
    //
	// Copies up to _qty elements from elmnt at posn _pos in queue into _dest
	// Returns actual no. of elements copied into memory indicated by dest.
	// Updates _pos to indicate 1st element (if any) not read from queue, allowing
	// reading queue in small steps.
	// Does NOT wrap around past end of queue
	//
	{
		if ( _pos >= count )
			return 0;		// _pos is outside valid queue range
	
		CircCntr	posn( start, _pos );				// posn of 1st elmnt to read
		uint		nofromPos = Count( _pos );
		//uint		nofromPos = end - posn;				// no. of elements from posn
		//nofromPos = nofromPos + _pos ? nofromPos : count;	// to the end of the queue
		
		if ( _qty > nofromPos )	// _qty is set to the no. of elements
			_qty = nofromPos;	// which can be read from posn _pos
		
		// now copy the requested elements from queue to dest
		uint	tmp = queSize - posn;
		if ( tmp >= _qty )
		{
			// simple, can do it in 1 move
			memmove( _dest, queue+posn, _qty * sizeof(T) );
		}
		else
		{
			// need to do move in 2 bites
			// bite 1. From posn to array physical end.
			memmove( _dest, queue+posn, tmp * sizeof(T) );
			// bite 2. From array physical start to end.
			memmove( _dest+tmp, queue, (_qty-tmp) * sizeof(T) );
		}
		
		posn += _qty;			// Update _pos so it refers to 1st elmnt (if any)
		_pos = posn - start;	// not read from queue
		return _qty;
	}

    uint   		ReadX( T* _dest, uint _qty, const uint& _pos )
    //
	// Copies up to _qty elements from elmnt at posn _pos in queue into _dest
	// Returns actual no. of elements read.
	// Does NOT wrap around past end of queue
	//
	{
		uint	pos = _pos;
		return Read( _dest, _qty, pos );
	}
	
	void		Empty() { count = 0; start = 0; end = 0; }

	uint		Count(uint _pos ) const { return count - _pos; }
	uint		Count()	const { return count; }
	uint		Capacity()	const { return queSize; }
	uint		FreeSpace() const { return queSize-count; }
	uint		End() const { return end; }
	uint		Start() const { return start; }

private:
    T*             	queue;      // ptr to the queue memory buffer
    uint          	queSize;    // no. of elemnts the queue can store
    uint			count;		// no. of elements in the queue
    CircCntr 		start;   	// ptr to 1st entry in queue
    CircCntr 		end;   		// ptr to next space in queue
};


#ifdef USING
}		// namespace ktUtil
#endif

//#include "bcirq_o.cpp"


#endif  // __BCIRCQ_O__HPP__
