//	CircQpov.hpp

#ifndef __CIRCQPOV__HPP__
#define __CIRCQPOV__HPP__

#include "CircQp.hpp"
#include "circq_o.hpp"

//===========================================================================//
//  class CircQuePtrOvr
//  Implements a simple circular queue, for pointers, allowing overwrites
//
//	Attempting to over fill the queue will overwrite existing data
//	Attempting to remove data from an empty queue will throw a CircQException
//
//  Public Methods:
//	---------------
//  constructor     Accepts 1 arg which specifies the queue size.
//                  Dynamically allocates memory for the queue buffer.
//  destructor      Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added.
//		If queue has no free space, 1st element in queue will be overwritten
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		If _qty is > free space in queue, data will be overwritten.
//		EG 1 free space = 10, _qty = 15, existing queue elmnts will be overwritten
//		EG 2 capacity = 10, _qty = 15, only last 10 elements in _src will be stored
//		Returns no. of elements added, does NOT throw CircQException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a CircQException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a CircQException
//
// T   		Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a CircQException
//
// T  		ReadX( const uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a CircQException
//		Wrapper for Read(). Can be called with constant for _pos
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//		Wrapper for Read(). Can be called with constant for _pos
//
// void		ClearUnused()
// 		resets all unused elements to 0.
// 		Provided for derived classes which store ptrs in queue.
// 		Setting ptr to 0 before deletion will prevent heap corruption
//
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
template<class T>
class   CircQuePtrOvr : public CircQuePtr<T>, private CirQ_o<T*>
{
public:
    CircQuePtrOvr( uint _size ) : CirQ_o(_size) {}
    virtual ~CircQuePtrOvr() {}
    
    virtual T*    	Add( T* _item ) { return CirQ_o::Add( _item ); }
    virtual uint	Add( T** _src, uint _qty ) { return CirQ_o::Add( _src, _qty ); }

    virtual T*   	Remove() { return CirQ_o::Remove(); }
    virtual uint   	Remove( T** _dest, uint _qty ) { return CirQ_o::Remove( _dest, _qty ); }

    virtual T*    	Read( uint& _pos )) { return CirQ_o::Read( _pos ); }
    virtual T*	    ReadX( const uint& _pos ) { return CirQ_o::ReadX( _pos ); }
    virtual uint   	Read( T** _dest, uint _qty, uint& _pos ) { return CirQ_o::Read( _dest, _qty, _pos ); }
    virtual uint   	ReadX( T** _dest, uint _qty, const uint& _pos ) { return CirQ_o::ReadX( _dest, _qty, _pos ); }

	virtual uint	Count( uint _pos ) const { return CirQ_o::Count( _pos ); }
	virtual uint	Count()	const { return CirQ_o::Count(); }
	virtual uint	Capacity()	const { return CirQ_o::Capacity(); }
	virtual uint	FreeSpace() const { return CirQ_o::FreeSpace(); }
	virtual uint	End() const { return CirQ_o::End(); }
	virtual uint	Start() const { return CirQ_o::Start(); }
};


#endif  // __CIRCQPOV__HPP__
