//  circq_o.hpp
//
//  Declaration of CircluarQueue class

#ifndef __CIRCQ_O__HPP__
#define __CIRCQ_O__HPP__

#include "CircCntr.hpp"


#ifndef  uint
typedef unsigned int  uint;
#endif

class	CircCntr;			// forward declaration
class	CircQException;		// forward declaration

//===========================================================================//
//  class CirQ_o
//  Implements a simple circular queue, allowing overwrites
//
//	Attempting to over fill the queue will overwrite existing data
//	Attempting to remove data from an empty queue will throw a CircQException
//
//  Public Methods:
//	---------------
//  constructor     Accepts 1 arg which specifies the queue size.
//                  Dynamically allocates memory for the queue buffer.
//  destructor      Deletes the memory buffer used by the queue
//
// const T& Add( const T& _item )
//		Adds a single element to the queue, returns the element added.
//		If queue has no free space, 1st element in queue will be overwritten
//
// void	 	Add( T* _src, uint _qty )
//		Adds _qty elements to queue, from array _src.
//		If _qty is > free space in queue, data will be overwritten.
//		EG 1 free space = 10, _qty = 15, existing queue elmnts will be overwritten
//		EG 2 capacity = 10, _qty = 15, only last 10 elements in _src will be stored
//		Returns no. of elements added, does NOT throw CircQException.
//
// const T&   	Remove()
//		Removes & returns a single element from the queue.
//		If queue is empty throws a CircQException 
//
// uint 	Remove( T* _dest, uint _qty )
//		Removes up to _qty elements from queue & places them into _dest.
//		Returns the actual no. of elements removed.
//		If queue is empty throws a CircQException
//
// T   		Read( uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Updates _pos to indicate next element (if any) in queue. NOTE _pos must be a var.
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a CircQException
//
// T  		ReadX( const uint& _pos )
// 		Returns copy of elmnt at posn _pos in queue
// 		Does Not wrap around past end of queue.
// 		If _posn indicates element outside queue, throws a CircQException
//		Wrapper for Read(). Can be called with constant for _pos
//
// uint		Read( T* _dest, uint _qty, uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		Updates _pos to indicate 1st element (if any) not read from queue, allowing
// 		reading queue in small steps. NOTE _pos must be a var.
// 		Does NOT wrap around past end of queue
//
// uint   	ReadX( T* _dest, uint _qty, const uint& _pos )
// 		Copies up to _qty elements from elmnt at posn _pos in queue into _dest
// 		Returns actual no. of elements read.
// 		reading queue in small steps.
// 		Does NOT wrap around past end of queue
//		Wrapper for Read(). Can be called with constant for _pos
//
// void		ClearUnused()
// 		resets all unused elements to 0.
// 		Provided for derived classes which store ptrs in queue.
// 		Setting ptr to 0 before deletion will prevent heap corruption
//
// uint		Count()		//	No. of elements currently in thew queue
// uint		Capacity()	//	No. of elements that the queue can hold
// uint		FreeSpace()	//	No. of elmnts that can be added w/o over writing data.
//
//===========================================================================//
template<class T>
class   CirQ_o
{
public:
    CirQ_o( uint _size );
    virtual ~CirQ_o();
    
    virtual const T&    Add( const T& _item );
    virtual uint	    Add( T* _src, uint _qty );

    virtual const T&   	Remove();
    virtual uint   		Remove( T* _dest, uint _qty );

    virtual const T&    Read( uint& _pos );
    virtual const T&    ReadX( const uint& _pos );
    virtual uint   		Read( T* _dest, uint _qty, uint& _pos );
    virtual uint   		ReadX( T* _dest, uint _qty, const uint& _pos );

	uint		Count(uint _pos ) const { return count - _pos; }
	uint		Count()	const { return count; }
	uint		Capacity()	const { return queSize; }
	uint		FreeSpace() const { return queSize-count; }
	uint		End() const { return end; }
	uint		Start() const { return start; }

//protected:
	void		ClearUnused();
	
private:
    T*             	queue;      // ptr to the queue memory buffer
    uint          	queSize;    // no. of elemnts the queue can store
    uint			count;		// no. of elements in the queue
    CircCntr 		start;   	// ptr to 1st entry in queue
    CircCntr 		end;   		// ptr to next space in queue
};


class	CircQException
{
public:
	enum	Flags
	{
		Ok			= 0,
		EmptyQ		= 1,
		OutsideQ	= 2,
		FullQ		= 3
	};
	
	CircQException( Flags _err=Ok ) : errCode(_err) {}
	
	uint	ErrCode() const { return errCode; }
	operator const char* () const { return ErrString[errCode]; }
private:
	Flags			errCode;		// 
	static char*	ErrString[];
};

char*	CircQException::ErrString[] = { "Ok", "Empty Queue", "Outside Queue", "Full Queue", 0 };

#include "circq_o.cpp"

#endif  // __CIRCQ_O__HPP__
